package com.mdd.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mdd.admin.service.ISystemAuthPermService;
import com.mdd.admin.service.ISystemAuthRoleService;
import com.mdd.admin.validate.commons.PageValidate;
import com.mdd.admin.validate.system.SystemRoleCreateValidate;
import com.mdd.admin.validate.system.SystemRoleUpdateValidate;
import com.mdd.admin.vo.system.SystemAuthRoleVo;
import com.mdd.common.core.PageResult;
import com.mdd.common.entity.system.SystemAuthRole;
import com.mdd.common.mapper.system.SystemAuthAdminMapper;
import com.mdd.common.mapper.system.SystemAuthRoleMapper;
import com.mdd.common.util.TimeUtils;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import javax.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

@Service
public class SystemAuthRoleServiceImpl implements ISystemAuthRoleService {
   @Resource
   SystemAuthAdminMapper systemAuthAdminMapper;
   @Resource
   SystemAuthRoleMapper systemAuthRoleMapper;
   @Resource
   ISystemAuthPermService iSystemAuthPermService;

   public List<SystemAuthRoleVo> all() {
      QueryWrapper<SystemAuthRole> queryWrapper = new QueryWrapper();
      queryWrapper.select(new String[]{"id,name,sort,is_disable,create_time,update_time"});
      queryWrapper.orderByDesc(Arrays.asList("sort", "id"));
      List<SystemAuthRole> systemAuthRoles = this.systemAuthRoleMapper.selectList(queryWrapper);
      List<SystemAuthRoleVo> list = new ArrayList();
      Iterator var4 = systemAuthRoles.iterator();

      while(var4.hasNext()) {
         SystemAuthRole systemAuthRole = (SystemAuthRole)var4.next();
         SystemAuthRoleVo vo = new SystemAuthRoleVo();
         vo.setId(systemAuthRole.getId());
         vo.setName(systemAuthRole.getName());
         vo.setSort(systemAuthRole.getSort());
         vo.setIsDisable(systemAuthRole.getIsDisable());
         vo.setCreateTime(TimeUtils.timestampToDate(systemAuthRole.getCreateTime()));
         vo.setUpdateTime(TimeUtils.timestampToDate(systemAuthRole.getUpdateTime()));
         vo.setMember(0);
         vo.setRemark("");
         vo.setMenus(Collections.EMPTY_LIST);
         list.add(vo);
      }

      return list;
   }

   public PageResult<SystemAuthRoleVo> list(@Validated PageValidate pageValidate) {
      Integer page = pageValidate.getPageNo();
      Integer limit = pageValidate.getPageSize();
      QueryWrapper<SystemAuthRole> queryWrapper = new QueryWrapper();
      queryWrapper.orderByDesc(Arrays.asList("sort", "id"));
      IPage<SystemAuthRole> iPage = this.systemAuthRoleMapper.selectPage(new Page((long)page, (long)limit), queryWrapper);
      List<SystemAuthRoleVo> list = new ArrayList();
      Iterator var7 = iPage.getRecords().iterator();

      while(var7.hasNext()) {
         SystemAuthRole systemAuthRole = (SystemAuthRole)var7.next();
         SystemAuthRoleVo vo = new SystemAuthRoleVo();
         BeanUtils.copyProperties(systemAuthRole, vo);
         List<Integer> ids = this.systemAuthAdminMapper.selectChildrenById(systemAuthRole.getId());
         Integer member = ids.size();
         vo.setMenus(new ArrayList());
         vo.setMember(member);
         vo.setCreateTime(TimeUtils.timestampToDate(systemAuthRole.getCreateTime()));
         vo.setUpdateTime(TimeUtils.timestampToDate(systemAuthRole.getUpdateTime()));
         list.add(vo);
      }

      return PageResult.iPageHandle(iPage.getTotal(), iPage.getCurrent(), iPage.getSize(), list);
   }

   public SystemAuthRoleVo detail(Integer id) {
      SystemAuthRole systemAuthRole = (SystemAuthRole)this.systemAuthRoleMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).eq("id", id)).last("limit 1"));
      Assert.notNull(systemAuthRole, "角色已不存在!", new Object[0]);
      List<Integer> roleIds = new LinkedList();
      roleIds.add(systemAuthRole.getId());
      SystemAuthRoleVo vo = new SystemAuthRoleVo();
      BeanUtils.copyProperties(systemAuthRole, vo);
      vo.setMember(0);
      vo.setMenus(this.iSystemAuthPermService.selectMenuIdsByRoleId(roleIds));
      vo.setCreateTime(TimeUtils.timestampToDate(systemAuthRole.getCreateTime()));
      vo.setUpdateTime(TimeUtils.timestampToDate(systemAuthRole.getUpdateTime()));
      return vo;
   }

   @Transactional
   public void add(SystemRoleCreateValidate createValidate) {
      Assert.isNull(this.systemAuthRoleMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,name"}).eq("name", createValidate.getName().trim())).last("limit 1")), "角色名称已存在!", new Object[0]);
      SystemAuthRole model = new SystemAuthRole();
      model.setName(createValidate.getName().trim());
      model.setRemark(createValidate.getRemark());
      model.setSort(createValidate.getSort());
      model.setIsDisable(createValidate.getIsDisable());
      model.setCreateTime(System.currentTimeMillis() / 1000L);
      model.setUpdateTime(System.currentTimeMillis() / 1000L);
      this.systemAuthRoleMapper.insert(model);
      this.iSystemAuthPermService.batchSaveByMenuIds(model.getId(), createValidate.getMenuIds());
   }

   @Transactional
   public void edit(SystemRoleUpdateValidate updateValidate) {
      Assert.notNull(this.systemAuthRoleMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,name"}).eq("id", updateValidate.getId())).last("limit 1")), "角色已不存在!", new Object[0]);
      Assert.isNull(this.systemAuthRoleMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,name"}).ne("id", updateValidate.getId())).eq("name", updateValidate.getName().trim())).last("limit 1")), "角色名称已存在!", new Object[0]);
      SystemAuthRole model = new SystemAuthRole();
      model.setId(updateValidate.getId());
      model.setName(updateValidate.getName().trim());
      model.setRemark(updateValidate.getRemark());
      model.setSort(updateValidate.getSort());
      model.setIsDisable(updateValidate.getIsDisable());
      model.setUpdateTime(System.currentTimeMillis() / 1000L);
      this.systemAuthRoleMapper.updateById(model);
      this.iSystemAuthPermService.batchDeleteByRoleId(updateValidate.getId());
      this.iSystemAuthPermService.batchSaveByMenuIds(updateValidate.getId(), updateValidate.getMenuIds());
   }

   @Transactional
   public void del(Integer id) {
      Assert.notNull(this.systemAuthRoleMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id", "name"}).eq("id", id)).last("limit 1")), "角色已不存在!", new Object[0]);
      Assert.isNull(this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id", "role", "nickname"}).eq("role", id)).eq("is_delete", 0)), "角色已被管理员使用,请先移除", new Object[0]);
      this.systemAuthRoleMapper.deleteById(id);
      this.iSystemAuthPermService.batchDeleteByRoleId(id);
   }
}
